    function [map, mapptr, mask] = rdis(A,ndom) 
%% -----------------------------------------------------------------------
%%      function [ndom, map, mapptr] = rdis(A,ndom) 
%%      recursive graph bisection algorithm for partitioning.
%%      initial graph is cut in two - then each time, the largest set
%%      is cut in two until we reach desired number of domains.
%% ----------------------------------------------------------------------- 
%%      input
%%      A         = matrix 
%%      ndom      = desired number of subgraphs
%%      output 
%%      ------
%%      map, mapptr  = pointer array data structure for domains. 
%%              if k1 = mapptr(i), k2=mapptr(i+1)-1 then 
%%              map(k1:k2) = points in domain number i
%%      mask  = array of size n -- mask(j)=subdomain number for node j. 
%%              [redondant information ] 
%% -----------------------------------------------------------------------;
      n = size(A,1) ; 
      idom = 1;
      siz(idom) = n;
      iptr(idom) = 1;
      mask(1:n) = 1 ;
%% 
%%      domain loop;
%% 
      while (idom  < ndom) 
%% 
%%      select domain with largest siz;
%%
      maxsiz = siz(1);
      bigdom = 1;
      for j=2:idom
          if (siz(j) > maxsiz) 
            maxsiz = siz(j);
            bigdom = j;
         end
      end
%% 
%%   do a prphn node traversal.. 
%%
      maskval = bigdom; 
      init = iptr(bigdom) ; 
      [nlev, levels, map] = perphn(A,init,mask,maskval);
%%
      iptr(idom) = map(1) ; 
      iptr(idom+1) = map(levels(nlev+1)-1) ;

%%d    for i=1:nlev
%%d      fprintf(' --\n') 
%%d      fprintf(' level %d \n', i) 
%%d      for j=levels(i):levels(i+1)-1 
%%d          fprintf(' %d ', map(j)) 
%%d      end
%%d    end 
%% 
%%   cut this domain in roughly two equal pieces 
%% 
      nextsiz = 0;
      wantsiz = maxsiz/2 ;
      idom = idom+1;
      for k = levels(nlev+1)-1:-1:1
          mask(map(k)) = idom;
          nextsiz = nextsiz+1 ; 
          if (nextsiz > wantsiz) 
             break; 
          end
      end
%%----------------------------------------------------------------------- 
     siz(idom) = nextsiz ; 
     siz(bigdom) = siz(bigdom) - nextsiz;
%%d     for k=1:n 
%%d        fprintf(' mask( %d ) =   %d \n',k,mask(k))
%%d     end
%%-----------------------------------------------------------------------
%%d     xx = [sum(mask==1),sum(mask==2),sum(mask==3),sum(mask==4)] 
%%d     fprintf(' =---------=\n'); 


%% 
%%      new initial point = last point of previous domain;
%% 
    end %% while 
%%
%%      domains found -- build data structure ;
%%
      mapptr(1) = 1;
      for idom=1:ndom 
         mapptr(idom+1) = mapptr(idom) + siz(idom) ;
      end
%% 
      for k=1:n
         idom = mask(k) ;
         ko = mapptr(idom) ;
         map(ko) = k;
         mapptr(idom) = ko+1;
      end
%% 
%%      reset pointers;
%% 
      for j= ndom:-1:1 
         mapptr(j+1) = mapptr(j) ;
      end
      mapptr(1) = 1 ;

